import pandas as pd
import numpy as np
import pystan
import matplotlib.pyplot as plt
import seaborn as sb
import scipy.stats as ss
import estmode as em

fin_nm = input('Input file (*.xlsx) = ')
fout_nm = input('Output file (*.txt) = ')
fout = open(fout_nm, 'w')
fout.write('Input Data File = {}\n'.format(fin_nm))

xlsx_fl = pd.ExcelFile(fin_nm)
data_xlsx = pd.read_excel(xlsx_fl,  header = None)
data = data_xlsx.values
var_name = data[0]
values = data[1:]

NSt = len(data) - 1
K = len(data[0]) - 1
Sts = np.empty(NSt)
Rs = np.empty((NSt, K), dtype = 'int')
for j in range(NSt):
    Sts[j] = data[j+1][0]
    for k in range(K):
        Rs[j][k] = data[j+1][k+1]

fout.write('\nData:\n')
for j in range(NSt):
    print(f'{Sts[j]:>10g}', end = '')
    fout.write(f'{Sts[j]:>10g}')
    for k in range(K):
        print(f'{Rs[j][k]:>5d}', end = '')
        fout.write(f'{Rs[j][k]:>5d}')
    print()
    fout.write('\n')

sm = pystan.StanModel(file = 'RatingEkman.stan')

Data = {'NSt':NSt, 'K':K, 'Sts':Sts, 'Rs':Rs}

fit = sm.sampling(data = Data, n_jobs = 1)

print(fit)

Psis = fit['Psis']
sgms = fit['sgms']
Cs = fit['C']
a = fit['a']
b = fit['b']

for i in range(NSt):
    sb.kdeplot(Psis.T[i], label = '$\psi_{}$'.format(i+1))
plt.title('Posterior Ditributions of $\psi_s$', fontsize = 20)
plt.yticks([])
plt.legend()
plt.tight_layout()
plt.savefig('FigPsiPost.png')
plt.show()

for i in range(NSt):
    sb.kdeplot(sgms.T[i], label = r'$\sigma_{}$'.format(i))
plt.title('Posterior Distributions of $\sigma_s$', fontsize = 20)
plt.legend()
plt.savefig('FigSgmPost.png')
plt.show()

plt.plot([0, 0], [0, 5], label = r'$C_0$')
for i in range(1, K-2):
    sb.kdeplot(Cs.T[i], label = r'$C_{}$'.format(i+1))
plt.plot([1,1], [0, 5], label = r'$C_{}$'.format(K-1))
plt.title('Posterior Distributions of Cs', fontsize = 20)
plt.yticks([])
plt.legend()
plt.savefig('FigCsPost.png')
plt.show()

mode_a = em.EstMode(a)
mode_b = em.EstMode(b)

plt.figure(figsize = (8,4))
plt.subplot(1,2,1)
sb.kdeplot(a)
plt.yticks([])
plt.xlabel('a', fontsize = 14)
plt.title('Posterior Distribution of a' +
          '\nmode = {0:.2f}'.format(mode_a),
          fontsize = 14)

plt.subplot(1,2,2)
sb.kdeplot(b)
plt.title('Posterior Distribtuion of b' +
          '\nmode = {0:.2f}'.format(mode_b), fontsize = 14)
plt.yticks([])
plt.xlabel('b', fontsize = 14)
plt.tight_layout()
plt.savefig('Fig_a_b_Post.png')
plt.show()

v_psis = np.empty(NSt)
v_sgms = np.empty(NSt)
for i in range(NSt):
    v_psis[i] = em.EstMode(Psis.T[i])
    v_sgms[i] = em.EstMode(sgms.T[i])

fout.write('\n{0:>15s}{1:>15s}{2:>15s}\n'.format('Stimulus', 'Mode Psi', 'Mode Sigma'))
for vst, vpsi, vsgm in zip(Sts, v_psis, v_sgms):
    fout.write('{0:>15.2f}{1:>15.2f}{2:>15.2f}\n'.format(vst, vpsi, vsgm))
    
v_Cs = np.empty(K-1)
v_Cs[0] = 0.0
v_Cs[K-2] = 1.0
for k in range(1, K-2):
    v_Cs[k] = em.EstMode(Cs.T[k])
for k in range(K-1):
    plt.plot([Sts[0], Sts[-1]], [v_Cs[k], v_Cs[k]], color = 'y')
plt.plot([], 'y-', label = 'C')
plt.plot(Sts, v_psis, color = 'b', lw = 3, label = '$\psi$')
plt.plot(Sts, v_psis - v_sgms, color = 'g', linestyle = '--', lw = 1,label = '$\psi - \sigma$')
plt.plot(Sts, v_psis + v_sgms, color = 'g', linestyle = '--', lw = 1, label = '$\psi+ \sigma$')
    
plt.xlabel('Stimulus', fontsize = 16)
plt.ylabel('$\psi$', fontsize = 16)
plt.legend(loc = 'upper left')
plt.title('Relation of Stimulus and $\psi$', fontsize = 18)
plt.tight_layout()
plt.savefig('PsiSgmC.png')
plt.show()

Probs = np.empty((NSt, K))
for s in range(NSt):
    Probs[s][0] = ss.norm(loc = v_psis[s], scale = v_sgms[s]).cdf(v_Cs[0])
    Probs[s][K-1] = 1.0 - ss.norm(loc = v_psis[s], scale = v_sgms[s]).cdf(v_Cs[K-2])
    for k in range(1, K-1):
        Probs[s][k] = ss.norm(loc = v_psis[s], scale = v_sgms[s]).cdf(v_Cs[k]) -\
                        ss.norm(loc = v_psis[s], scale = v_sgms[s]).cdf(v_Cs[k-1])
            
print('Predicted Prob(Rating|Stimulus):')
for s in range(NSt):
    for k in range(K):
        print(f'  {Probs[s][k]:7.3f}', end = '')
    print()
    
Pred_Rs = np.empty((NSt, K))
N_Rs = np.sum(Rs, axis = 1)
for i in range(NSt):
    Pred_Rs[i] = N_Rs[i] * Probs[i]
    
print('Predicted Frequencies of Rating for Stimulus:')
for s in range(NSt):
    for k in range(K):
        print(f'  {Pred_Rs[s][k]:7.1f}', end = '')
    print()

cum_data_Rs = np.cumsum(Rs, axis = 1)
cum_pred_Rs = np.cumsum(Pred_Rs, axis = 1)

print(cum_data_Rs)

for s in range(NSt):
    if s == 0:
        plt.plot(range(K), cum_data_Rs[s]/cum_data_Rs[s][K-1], 'g:', lw = 3, label = 'Data')
    else:
        plt.plot(range(K), cum_data_Rs[s]/cum_data_Rs[s][K-1], 'g:', lw = 3 )
    if s == 0:
        plt.plot(range(K), cum_pred_Rs[s]/cum_pred_Rs[s][K-1], 'b-', label = 'Model')
    else:
        plt.plot(range(K), cum_pred_Rs[s]/cum_pred_Rs[s][K-1], 'b-')

plt.xticks(range(K), range(1, K+1))
plt.xlabel('Rating', fontsize = 14)
plt.ylabel('Cum. Prop.', fontsize = 14)
plt.legend()
plt.title('Data and Model Prediction', fontsize = 18)
plt.tight_layout()
plt.savefig('FigDataModel.png')
plt.show()

fout.close()
print('\n' + fout_nm + ' was saved.\n')
